#include <SDL/SDL_gfxPrimitives.h>
#include <math.h>

#include "utils.h"
#include "globe.h"


void 
SNOW::randomize(
	int max_r,
	int rand_r
)
{
	double r = (rand_r ? RANDD((double) max_r) : max_r);
	double d = RANDD(M_PI);

	x = (int) r * cos(d);
	y = (int) -r * sin(d);
}

int
SNOW::out_of_range(
	double max_r
)
{
	return (sqrt(y * y + x * x) > max_r);
}

//////////////////////////////////////////////////////////////////////


GLOBE::GLOBE(
	GRAPHICS *graphics
)
{
	_graphics = graphics;

	_base_surf = _graphics->load_image("resources/snow_globe_bases.png");
	_dome_surf = _graphics->load_image("resources/snow_globe_domes.png");
	_snow_surf = _graphics->load_image("resources/snow.png");
	_hand_surf = _graphics->load_image("resources/hand.png");

	_perspective = P_SIDE;

	_snow_speed   =  12.0;
	_bubble_speed = -36.0;

	_hand_state = 0;
	_hand_pos = 0;
	_hand_rate = 10;

	_offset_x = 16;
	_offset_y = 0;

	//////////

	_world = new WORLD(_graphics);

	//////////

	// create snow
	for(int i = 0; i < 200; i++) {
		SNOW *s = new SNOW();

		s->randomize(_base_surf->w / 2 - 16, 1);

		// printf("snow (%d, %d)\n", s->x, s->y);

		s->type = RANDI(6);
		s->angle = RANDD(2*M_PI);

		_snow.push_front(s);
	}

}

GLOBE::~GLOBE(
)
{
	// FIXME: verify they were all allocated, otherwise don't free...
	SDL_FreeSurface(_base_surf);
	SDL_FreeSurface(_dome_surf);
	SDL_FreeSurface(_snow_surf);
	SDL_FreeSurface(_hand_surf);
}

////////////////////////////////////////

void 
GLOBE::update(
	float dt
)
{
	_world->update(dt);

	//////////

	// update snow

	std::list<SNOW *>::iterator s_it;

	for(s_it = _snow.begin(); s_it != _snow.end(); s_it++) {
		(*s_it)->angle += M_PI / 32;

		(*s_it)->y += _snow_speed * dt;

		if ((*s_it)->y >= 0) {
			(*s_it)->y = 0;

			if	(RANDI((int) (20.0 / dt)) == 0) {
				(*s_it)->randomize(_base_surf->w / 2 - 16, 0);
			}
		}
		else {
			(*s_it)->x += .7 * sin((*s_it)->angle);
		}
	}

	// update bubbles

	std::list<SNOW *>::iterator b_it;

	for(b_it = _bubbles.begin(); b_it != _bubbles.end(); b_it++) {
		(*b_it)->angle += M_PI / 32;

		(*b_it)->y += _bubble_speed * dt;

		if ((*b_it)->out_of_range(_base_surf->w / 2  - 16)) {
			(*b_it)->type = -1;
		}
		else {
			(*b_it)->x += .7 * sin((*b_it)->angle);
		}
	}

	// FIXME: O(n^2) time, but should be small, so leave it.
	for(b_it = _bubbles.begin(); b_it != _bubbles.end(); b_it++) {
		if ((*b_it)->type == -1) {
			_bubbles.erase(b_it);
			b_it = _bubbles.begin();
		}
	}

	// update hand

	if (_hand_state) {
		_hand_pos += _hand_rate;

		// How far we need to go, is relative to offset
		if (_hand_pos > (300 - _offset_x)) {
			_hand_pos = (300 - _offset_x);
		}
	}
	else {
		if (_hand_pos > 0) {
			_hand_pos -= _hand_rate;
		}
	}
}


void 
GLOBE::draw(
	void
)
{
	//////////

	SDL_Rect srect;
	SDL_Rect drect;

	//////////

	// globe base
	srect.x = 0;
	srect.y = 0;
	srect.w = _base_surf->w;
	srect.h = 300;

	drect.x = ( _graphics->primary->w - srect.w ) / 2 + _offset_x;
	drect.y = _offset_y;
	drect.w = srect.w;
	drect.h = srect.h;

	SDL_BlitSurface(_base_surf, &srect, _graphics->primary, &drect);

	//////////

	_world->draw(_offset_x, _offset_y);

	//////////

	// snow
	srect.x = 0;
	srect.y = 0;
	srect.w = 9;
	srect.h = 9;
	drect.w = 9;
	drect.h = 9;

	std::list<SNOW *>::iterator s_it;

	for(s_it = _snow.begin(); s_it != _snow.end(); s_it++) {
		srect.x = ((*s_it)->type % 4) * 9;
		srect.y = (int) ((*s_it)->type / 4) * 9;

		drect.x = (int) ((*s_it)->x - 4) + _offset_x + _graphics->primary->w/2;
		drect.y = (int) ((*s_it)->y - 4) + _offset_y + 300 - 50;

		// pixelColor(_graphics->primary, (*s_it)->x, (*s_it)->y, 0xFF000088);

		SDL_BlitSurface(_snow_surf, &srect, _graphics->primary, &drect);
	}

	//////////

	// bubbles
	srect.x = 0;
	srect.y = 0;
	srect.w = 9;
	srect.h = 9;
	drect.w = 9;
	drect.h = 9;

	std::list<SNOW *>::iterator b_it;

	for(b_it = _bubbles.begin(); b_it != _bubbles.end(); b_it++) {
		srect.x = ((*b_it)->type % 4) * 9;
		srect.y = (int) ((*b_it)->type / 4) * 9;

		drect.x = (int) ((*b_it)->x - 4) + _offset_x + _graphics->primary->w/2;
		drect.y = (int) ((*b_it)->y - 4) + _offset_y + 300 - 50;

		SDL_BlitSurface(_snow_surf, &srect, _graphics->primary, &drect);
	}

	//////////

	// globe
	srect.x = 0;
	srect.y = 0;
	srect.w = _base_surf->w;
	srect.h = 300;

	drect.x = ( _graphics->primary->w - srect.w ) / 2 + _offset_x;
	drect.y = _offset_y;
	drect.w = srect.w;
	drect.h = srect.h;

	SDL_BlitSurface(_dome_surf, &srect, _graphics->primary, &drect);

	//////////

	// hand
	if (_hand_pos > 0) {

		srect.x = 0;
		srect.y = 0;
		srect.w = _hand_surf->w;
		srect.h = 300;

		drect.x = _graphics->primary->w - _hand_pos;
		drect.y = _offset_y;
		drect.w = srect.w;
		drect.h = srect.h;

		SDL_BlitSurface(_hand_surf, &srect, _graphics->primary, &drect);
	}

}


void 
GLOBE::hand_start(
	void
)
{
	_hand_state = 1;
}


void 
GLOBE::hand_stop(
	void
)
{
	_hand_state = 0;
}

int  
GLOBE::full_hand(
	void
)
{
	return (_hand_pos >= (300 - _offset_x));
}


void 
GLOBE::shake(
	int dx,
	int dy
)
{
	// printf("shake\n");

	_offset_x += dx;
	_hand_pos -= dx;

	_world->shake(dx, (_perspective == P_TOP) ? dy : 0);

	//////////

	if (((_perspective == P_SIDE) && (dx != 0)) ||
	    (_perspective != P_SIDE)) {

		std::list<SNOW *>::iterator s_it;

		for(s_it = _snow.begin(); s_it != _snow.end(); s_it++) {
			if (((*s_it)->y >= -32) && (RANDI(3) == 0)) {
				int r = ((*s_it)->y < -24) ? 4 : 3;

				(*s_it)->y -= -1 + RANDI(4);

				if ((*s_it)->out_of_range( _base_surf->w / 2  - 16)) {
					(*s_it)->randomize(_base_surf->w / 2 - 16, 0);
				}
			}

			if ((*s_it)->y >= 0) {
				(*s_it)->y = 0;
			}
		}
	}

	//////////

	// It's easy to be shaken quite a bit...
	if (RANDI(3) == 0) {
		SNOW *b = new SNOW();

		b->randomize(_base_surf->w / 2 - 16, 1);

		// printf("snow (%d, %d)\n", s->x, s->y);

		b->type = 8 + RANDI(3);
		b->angle = RANDD(2*M_PI);

		_bubbles.push_front(b);
	}
}


void 
GLOBE::stir_snow(
	void
)
{
	;
}


